#include <Windows.h>
#include <string>
#include <deque>
#include "resource.h"

namespace{

  HWND hDlgGlob = NULL;
  HWND hWndGlob = NULL;

  struct ImageInfo{
    std::wstring mImagePath;
  };

  class Images{
  public:
    Images(void)
      : mImages(){
    }

    ~Images(void) throw() {}

    void addImage(std::wstring const &image){
      mImages.push_back(image);
    }

    std::size_t getSize(void) const{
      return mImages.size();
    }

    std::wstring const &getImage(int index) const{
      if (index < 0 || index >= mImages.size())
        return L"";

      return mImages[index];
    }

  private:
    std::deque<std::wstring> mImages;
  };

  Images images;
}

LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DialogProc(HWND, UINT, WPARAM, LPARAM);
void drawPicture(HWND hWnd, std::wstring const &image);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow){
  WNDCLASS WndClass = {0};
  WndClass.style = CS_HREDRAW | CS_VREDRAW;
  WndClass.lpfnWndProc = MainWndProc;
  WndClass.cbClsExtra = 0;
  WndClass.cbWndExtra = 0;
  WndClass.hInstance = hInstance;
  WndClass.hIcon = LoadIcon(NULL,IDI_APPLICATION);
  WndClass.hCursor = LoadCursor(NULL,IDC_ARROW);
  WndClass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
  WndClass.lpszMenuName = NULL;

  WCHAR className[] = L"ClientAppClass";
  WndClass.lpszClassName = className;

  if(!RegisterClass(&WndClass)){
    MessageBox(NULL, L"Can't register class", L"Error", MB_OK);
    return 0;
  }

  WCHAR windowTitle[] = L"ClientAppTitle";

  HWND hWnd = CreateWindowEx(WS_EX_TOPMOST, className, windowTitle, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
  if(!hWnd){
    MessageBox(NULL, L"Can't create window", L"Error", MB_OK);
    return 0;
  }

  hWndGlob = hWnd;

  HWND hDialog = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), 0, (DLGPROC) DialogProc);
  
  hDlgGlob = hDialog;
  ShowWindow(hDialog, SW_SHOW);
  UpdateWindow(hDialog);

  MSG message;
  while(GetMessage(&message, NULL, 0, 0)){
    TranslateMessage(&message);
    DispatchMessage(&message);
  }

  return (int) message.wParam;
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam){
  switch (Message){
    case WM_COPYDATA:
      SendMessage(hDlgGlob, WM_COPYDATA, wParam, lParam);
      return 0;

    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
  }

  return DefWindowProc(hWnd, Message, wParam, lParam);
}

void updateStatics(HWND hDlg, int position, int count, std::wstring const &path){
  static WCHAR buff[32] = {0};
  swprintf(buff, 32, L"%i/%i", position + 1, count);
  SetWindowText(GetDlgItem(hDlg, IDC_STATIC_PAGES), buff);
  SetWindowText(GetDlgItem(hDlg, IDC_STATIC_PATH), path.c_str());
}

BOOL CALLBACK DialogProc(HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam){

  static int currentImage = -1;
  

  switch(Message){
    case WM_INITDIALOG:{
      break;
    }

    case WM_COMMAND:{
      if (HIWORD(wParam) == BN_CLICKED){
        if (LOWORD(wParam) == IDC_BUTTON_LEFT){
          if (currentImage <= 0)
            return 0;

          --currentImage;
          updateStatics(hDlg, currentImage, images.getSize(), images.getImage(currentImage));
          InvalidateRect(hDlg, NULL, TRUE);
          return 0;
        }
        else if (LOWORD(wParam) == IDC_BUTTON_RIGHT){
          if (currentImage + 1 >= images.getSize())
            return 0;

          ++currentImage;
          updateStatics(hDlg, currentImage, images.getSize(), images.getImage(currentImage));
          InvalidateRect(hDlg, NULL, TRUE);
          return 0;
        }
      }

      return 0;
    }

    case WM_PAINT:{
      PAINTSTRUCT ps = {0};
      BeginPaint(hDlg, &ps);
      drawPicture(hDlg, images.getImage(currentImage));
      EndPaint(hDlg, &ps);
      return 0;
    }

    case WM_COPYDATA:{
      COPYDATASTRUCT *data = (COPYDATASTRUCT *) lParam;
      WCHAR const *package = (WCHAR const *) data->lpData;
      std::wstring fileName(package);

      std::size_t index = fileName.rfind('.');
      std::wstring extension = L"";
      if (index != std::string::npos)
        extension = fileName.substr(index + 1);

      if (!_wcsicmp(extension.c_str(), L"ini")){ // INI file received
      }
      else{ // paths to converted files received
        package += wcslen(package) + 1; // skip header "printer name\0"
        package += wcslen(package) + 1; // skip header "job id\0"

        WCHAR const *ptr = package;
        while (ptr[0]){
          fileName = ptr;
          index = fileName.rfind('.');
          if (index != std::string::npos){
            extension = fileName.substr(index + 1);
            if (!_wcsicmp(extension.c_str(), L"bmp")){ // found BMP file
              images.addImage(fileName);
              updateStatics(hDlg, currentImage, images.getSize(), images.getImage(currentImage).c_str());
              InvalidateRect(hDlg, NULL, TRUE);
              return 0;
            }
          }

          ptr += wcslen(ptr) + 1;
        }
      }

      return 0;
    }

    case WM_CLOSE:
      DestroyWindow(hDlg);
      break;

    case WM_DESTROY:
      PostQuitMessage(0);
      break;
  }

  return 0;
}

void drawPicture(HWND hWnd, std::wstring const &image){
  HWND imageControl = GetDlgItem(hWnd, IDC_PIC_CTRL);
  HDC hDC = GetDC(imageControl);
  HANDLE hImage = LoadImage(NULL, image.c_str(), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

  BITMAP Bitmap;
  GetObject(hImage, sizeof(BITMAP), &Bitmap);
  HDC hCompatibleDC = CreateCompatibleDC(hDC);
  HANDLE hOldBitmap = SelectObject(hCompatibleDC, hImage);

  RECT Rect = {0};
  GetClientRect(imageControl, &Rect);
  StretchBlt(hDC, 0, 0, Rect.right, Rect.bottom, hCompatibleDC, 0, 0, Bitmap.bmWidth, Bitmap.bmHeight, SRCCOPY);
  SelectObject(hCompatibleDC, hOldBitmap);
  DeleteDC(hCompatibleDC);
  DeleteObject(hImage);
  ReleaseDC(imageControl, hDC);
}
	